#include <windows.h>
#include <memory.h>
#include <commdlg.h>
#include "global.h"
#include "print.h"

//****************************************************************************
// Function: GetDefaultPrinterDC
//
// Purpose: Gets a printer DC to the default printer.
//
// Parameters:
//    hWnd - handle to the top level app window.
//
// Returns : None
//
// History:  Date       Author        Reason
//           5/15/93                   Created
//****************************************************************************

HDC GetDefaultPrinterDC(HWND hWnd)
{
	PRINTDLG pd;
		
	_fmemset((LPPRINTDLG)&pd, 0, sizeof(PRINTDLG));
	pd.lStructSize = sizeof(PRINTDLG);
	pd.hwndOwner = hWnd;
	pd.Flags = PD_RETURNDEFAULT | PD_RETURNDC;
	
	PrintDlg(&pd);
	
    if (pd.hDevMode != NULL)
       GlobalFree(pd.hDevMode);
    if (pd.hDevNames != NULL)
       GlobalFree(pd.hDevNames);
       
	return pd.hDC;
}

//****************************************************************************
// Function: PrintHandler
//
// Purpose: Prints the document.
//
// Parameters:
//    hWnd - handle to the top level app window.
//    hPrnDC - DC to the printer.
//
// Returns : None
//
// History:  Date       Author        Reason
//           5/15/93                   Created
//****************************************************************************
      
void PrintHandler(HWND hWnd, HDC hPrnDC)			
{
	FARPROC lpAbortProc;
	DOCINFO di;
	UINT uPageNum = 0;
	
	lpAbortProc = MakeProcInstance(PrintAbortProc, ghInst);

	/* Define the abort function */
	SetAbortProc(hPrnDC, lpAbortProc);
		
	di.cbSize = sizeof(DOCINFO);
	di.lpszDocName = (LPCSTR)"MULFON document";
	di.lpszOutput = NULL;
        
	if (SP_ERROR == StartDoc(hPrnDC, (LPDOCINFO)&di))
		MessageBox(hWnd, "Unable to start print job", NULL, MB_OK | MB_ICONHAND);
	else
	{
		for (uPageNum = 0; uPageNum < guNumPages; uPageNum++)
		{	
			StartPage(hPrnDC);
			SelectObject(hPrnDC, (HGDIOBJ)ghFont);
			RepaintClient(NULL, hPrnDC, glpPages[uPageNum].uFirstLine, 
			              glpPages[uPageNum].uLastLine);			
			EndPage(hPrnDC);
		}	
		EndDoc(hPrnDC);
		SelectObject(hPrnDC, (HGDIOBJ)ghFont);
	}		
		
	FreeProcInstance(lpAbortProc);
}

/****************************************************************
 * PrintAbortProc() is the abort procedure used by Windows when *
 * printing.                                                    *
 * Return TRUE indicates to keep printing                       *
 ****************************************************************/

BOOL FAR PASCAL PrintAbortProc(HDC hdcPrn, short nCode)
{
   MSG msg;

   while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
   {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }
   return TRUE;
}

//****************************************************************************
// Function: PrintPreviewHandler
//
// Purpose: Sets up the environment for Print Preview.
//
// Parameters:
//    hWnd - handle to the top level app window.
//    hPrnDC - DC to the printer.
//
// Returns : None
//
// History:  Date       Author        Reason
//           5/15/93                   Created
//****************************************************************************

void PrintPreviewHandler(HWND hWnd, HDC hPrnDC)
{
	HWND hPrevWnd;
	WNDCLASS wc;
	RECT rc;
	
	wc.style = CS_HREDRAW |
              CS_VREDRAW;
	wc.lpfnWndProc = PrevWndProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = ghInst;
	wc.hIcon = NULL;
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = COLOR_WINDOW + 1;
	wc.lpszMenuName = NULL;
	wc.lpszClassName = "PrevWndClass";

	// Register the window class and return success/failure code.

	RegisterClass(&wc);
	
	GetWindowRect(GetDesktopWindow(), (LPRECT)&rc);
	hPrevWnd = CreateWindow("PrevWndClass", "Print Preview of textfile.txt",
				            WS_POPUPWINDOW | WS_CAPTION | WS_MAXIMIZE, 
				            0, 0, rc.right, rc.bottom,
							hWnd, NULL, ghInst, NULL);
							
	ShowWindow(hPrevWnd, SW_SHOWMAXIMIZED);
	UpdateWindow(hPrevWnd); 						
}

//****************************************************************************
// Function: PrevWndProc
//
// Purpose: Displays the document in the print preview window.
//
// Parameters:
//    Standard window procedure parameters.
//
// Message Processing:

//		WM_CREATE: 
//			1) Creates buttons to browse through pages in the document.
//			2) Computes the margins of the document.

//      WM_COMMAND:
//          1) Handles the requests from the controls.
//
//      WM_PAINT:                                        
//			1) Sets the appropriate scaling for Print Preview.
//			2) Draws the print preview pages and borders.
//
// Comments:
//
//	All the Print Preview work has been accomplished with the help of GDI's
//	mapping mode and window/viewport extent support.
//
// Returns : standard window proc return.
//
// History:  Date       Author        Reason
//           5/15/93                   Created
//****************************************************************************

long FAR PASCAL PrevWndProc (HWND hWnd, unsigned message, WORD wParam, LONG lParam)
{
	static HWND hWndLeft, hWndRight, hWndPages;
	static RECT rc;
	PAINTSTRUCT ps;
	static UINT uLeftPage, uRightPage;
	static POINT ptOrigin, ptPhyPage, ptDocPage;
	static char szPages[10];
	
	switch (message)
	{
		case WM_CREATE:
			uLeftPage = 1;
			uRightPage = 2;
			GetClientRect(hWnd, (LPRECT)&rc);
			
			// create the flip left and flip right buttons
			hWndLeft = CreateWindow("Button", "<--",
				            		WS_CHILD | WS_BORDER | WS_VISIBLE, 
				            		rc.right/2 - 100, 10, 50, 30,
									hWnd, NULL, ghInst, NULL);
			EnableWindow(hWndLeft, FALSE);						
			hWndPages = CreateWindow("Static", "",
				            		WS_CHILD | WS_BORDER | SS_CENTER | WS_VISIBLE, 
				            		rc.right/2 - 35, 10, 70, 30,
									hWnd, NULL, ghInst, NULL);									
			hWndRight = CreateWindow("Button", "-->",
				            		WS_CHILD | WS_BORDER | WS_VISIBLE, 
				            		rc.right/2 + 50, 10, 50, 30,
									hWnd, NULL, ghInst, NULL);
									
			// compute the margins and other needed metrics.
			if (Escape(ghPrnDC, GETPHYSPAGESIZE, NULL, NULL, (LPPOINT)&ptPhyPage) == 0)
			{
				ptPhyPage.x = MulDiv(85, gnPrnXRes, 10);
				ptPhyPage.y = 11*gnPrnYRes;
			}
			if (Escape(ghPrnDC, GETPRINTINGOFFSET, NULL, NULL, (LPPOINT)&ptOrigin) == 0)
			{
				ptOrigin.x = gnPrnXRes/2;
				ptOrigin.y = gnPrnYRes/2;
			}	
			ptDocPage.x = GetDeviceCaps(ghPrnDC, HORZRES);																	
			ptDocPage.y = GetDeviceCaps(ghPrnDC, VERTRES);
			break;
			
		case WM_COMMAND:
			if (LOWORD(lParam) == hWndLeft && HIWORD(lParam) == BN_CLICKED)
			{
				uLeftPage -= 2;
				uRightPage = uLeftPage + 1;
				EnableWindow(hWndRight, TRUE);
				if (uLeftPage == 1)
					EnableWindow(hWndLeft, FALSE);
				InvalidateRect(hWnd, NULL, TRUE);
				UpdateWindow(hWnd);	
			}
			else if (LOWORD(lParam) == hWndRight && HIWORD(lParam) == BN_CLICKED)
			{
				uLeftPage += 2;
				uRightPage = uLeftPage + 1;
				EnableWindow(hWndLeft, TRUE);
				InvalidateRect(hWnd, NULL, TRUE);
				UpdateWindow(hWnd);				
			}
			break;	
			
		case WM_PAINT:
		{
			HFONT hOldFont;
			int iBot;
			POINT pt;
			
			BeginPaint(hWnd, (LPPAINTSTRUCT)&ps);
			SetBkMode(ps.hdc, TRANSPARENT);
			SetMapMode(ps.hdc, MM_ANISOTROPIC);
            
            // Draw the border of the two pages
			iBot = MulDiv(rc.right/2, ptPhyPage.y, ptPhyPage.x);
			MoveTo(ps.hdc, 0, 50);
			LineTo(ps.hdc, rc.right, 50);
			LineTo(ps.hdc, rc.right, 50+iBot);
			LineTo(ps.hdc, 0, 50+iBot);
			LineTo(ps.hdc, 0, 50);
			MoveTo(ps.hdc, rc.right/2, 50);
			LineTo(ps.hdc, rc.right/2, 50+iBot);
			
			// Logical window has the dimensions of the printer page
			SetWindowExt(ps.hdc, ptPhyPage.x, ptPhyPage.y);
			// Physical window uses has half the client area to represent one page in the
			// x-axis and a proportionate amount in the y-axis.						
			SetViewportExt(ps.hdc, rc.right/2, iBot);
			
			pt.x = ptOrigin.x;
			pt.y = ptOrigin.y;
			LPtoDP(ps.hdc, (LPPOINT)&pt, 1);
			// set the origin of printable text in the print preview window. The coordinates
			// in pt have the scaled margins. The constant 50 added in the y-axis accounts
			// for the space used by the button controls in the print preview window.
			SetViewportOrg(ps.hdc, pt.x, 50+pt.y);
			hOldFont = (HFONT)SelectObject(ps.hdc, (HGDIOBJ)ghFont);

			if (uRightPage > guNumPages)
			{
				EnableWindow(hWndRight, FALSE);
				uRightPage--;
				wsprintf(szPages, "%u - ", uLeftPage);
			}
			else
				wsprintf(szPages, "%u - %u", uLeftPage, uRightPage);
			
			SetWindowText(hWndPages, szPages);
			
			// Draw the left hand side page
			RepaintClient(ps.hdc, ghPrnDC, glpPages[uLeftPage-1].uFirstLine, glpPages[uLeftPage-1].uLastLine);
            
            // Draw the page margin for the left page
			DrawMargins(ps.hdc, (LPPOINT)&ptDocPage);
			if (uRightPage > uLeftPage)
			{   
				// let the second page be painted on the second leaf of the open book
				SetViewportOrg(ps.hdc, rc.right/2 + pt.x, 50+pt.y);
				// Draw the right hand side page
				RepaintClient(ps.hdc, ghPrnDC, glpPages[uRightPage-1].uFirstLine, glpPages[uRightPage-1].uLastLine);			
				// draw margins
				DrawMargins(ps.hdc, (LPPOINT)&ptDocPage);
			}
			
			SelectObject(ps.hdc, (HGDIOBJ)hOldFont);			
			EndPaint(hWnd, (LPPAINTSTRUCT)&ps);		
			break;
		}				
			
		case WM_CLOSE:
			DestroyWindow(hWnd);
			break;
			
		// We didn't handle, pass to DefWindowProc.
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return (NULL);
}

//****************************************************************************
// Function: DrawMargins
//
// Purpose: Draws a discontinued line around text to separate the printable 
//          area from the non-printable area of the physical page.
//
// Parameters:
//    hDC - DC to the print preview window.
//    lpptDims - Top left and bottom right coordinates of the printable portion
//               of the page.
//
// Returns : None
//
// History:  Date       Author        Reason
//           5/15/93                   Created
//****************************************************************************

void DrawMargins(HDC hDC, LPPOINT lpptDims)
{
	HPEN hPen, hOldPen;
	
	hPen = CreatePen(PS_DASH, 0, RGB(0, 0, 0));
	hOldPen = (HPEN)SelectObject(hDC, (HGDIOBJ)hPen);
	MoveTo(hDC, 0, 0);
	LineTo(hDC, lpptDims->x, 0);
	LineTo(hDC, lpptDims->x, lpptDims->y);
	LineTo(hDC, 0, lpptDims->y);
	LineTo(hDC, 0, 0);
	hPen = (HPEN)SelectObject(hDC, (HGDIOBJ)hOldPen);
	DeleteObject((HGDIOBJ)hPen);
}